package vc.thinker.absctacts.protocolproxy.protocol;

import com.google.common.collect.Lists;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import vc.thinker.absctacts.protocolproxy.dto.IMeterReportEntity;
import vc.thinker.absctacts.protocolproxy.dto.IServiceExecuteCmd;
import vc.thinker.absctacts.protocolproxy.dto.IServiceExecuteResult;
import vc.thinker.absctacts.protocolproxy.dto.ServiceExecuteResult;
import vc.thinker.absctacts.protocolproxy.enums.IMeterAttrDefine;
import vc.thinker.absctacts.protocolproxy.enums.IMeterEventDefine;
import vc.thinker.mqtt.proxyclient.entity.up.MeterData;
import vc.thinker.mqtt.proxyclient.entity.up.ThirdPartyProxyReport;
import vc.thinker.mqtt.proxyclient.service.ThirdPartyProxyClientService;
import vc.thinker.utils.LocalDateTimeUtils;

import java.util.List;
import java.util.stream.Collectors;

/**
 * 第三方协议
 *
 * @author HeTongHao
 * @since 2021/7/9 16:20
 */
@Slf4j
public abstract class AbstractThirdPartyProtocol implements IThirdPartyProtocol {
    @Autowired
    protected ThirdPartyProxyClientService thirdPartyProxyClientService;

    @Override
    public void report(List<IMeterReportEntity> meterReportEntityList) {
        thirdPartyProxyClientService.report(new ThirdPartyProxyReport(protocolType().getCode()
                        , meterReportEntityList.stream()
                        .map(this::meterReportEntityToMeterData)
                        .collect(Collectors.toList())
                )
        );
    }

    /**
     * 单组数据上报
     *
     * @param meterReportEntity meterReportEntity
     */
    public void report(IMeterReportEntity meterReportEntity) {
        report(Lists.newArrayList(meterReportEntity));
    }

    private MeterData meterReportEntityToMeterData(IMeterReportEntity meterReportEntity) {
        MeterData meterData = new MeterData()
                .setDeviceCode(meterReportEntity.getDeviceCode())
                .setTimestamp(LocalDateTimeUtils.parseToTimestamp(meterReportEntity.getCollectTime()));
        meterReportEntity.getDataMap().forEach((dataDefine, value) -> {
            if (dataDefine instanceof IMeterAttrDefine) {
                IMeterAttrDefine meterAttrDefine = (IMeterAttrDefine) dataDefine;
                meterData.putAttrData(meterAttrDefine.getCode(), value);
            } else if (dataDefine instanceof IMeterEventDefine) {
                IMeterEventDefine meterEventDefine = (IMeterEventDefine) dataDefine;
                if (value instanceof Boolean) {
                    value = (Boolean) value ? 1 : 0;
                }
                meterData.putEventData(meterEventDefine.getCode(), value);
            }
        });
        return meterData;
    }

    @Override
    public IServiceExecuteResult onExecutiveService(IServiceExecuteCmd cmd) {
        return ServiceExecuteResult.fail(cmd, "未实现任何服务");
    }
}
